package business;

import rmi.*;
import network.ListenerCommunicator;
import access.DatabaseConstants;
import access.Database;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.File;


public class   ListenerManager implements DatabaseConstants
{
    static private RegisterManager objRM;

    static private SoundSynthesizer objSS;
    private ListenerCommunicator objLC;

    //static private Vector[] vVector= null;
    static private Vector[] vRecordedVector= null;
    static private boolean bPlayRecordedProgram=false;
    static private String sRecordedProgramName=null;

    static private Hashtable htRecordedPrograms = new Hashtable();


    static private Vector vProgramsBuffer = new Vector();
    static private Vector vEventsBuffer= new Vector();
    static private boolean bStatus = false;

    /**
     *
     */
    public ListenerManager()
    {
        Database objDB = new Database();
        objDB.createFolders();
        System.out.println("before register manager");
        objRM = new RegisterManager();
        System.out.println("after register manager");
        objSS = new SoundSynthesizer();
        System.out.println("after sound synthesiser");
        try
        {
            objLC = new ListenerCommunicator();
        }
        catch(Exception re)
        {
            re.printStackTrace();
            System.out.println("Remote Exception ");
        }

        if( objLC.initialise() == false)
        {
            //get data from GUI;
            //update data from gui and call initialise.
        }
        initialiseRecorders();
    }

    public void updateProgramMapping(String sProgramName, Vector vEventNames, Vector vEventSoundProperties)
    {
       objRM.updateProgramMapping(sProgramName, vEventNames, vEventSoundProperties);
    }

    public void changeEventSoundMapping(String sProgramName, String sEventName, Vector vSoundProperties)
    {
        objRM.changeEventSoundMapping(sProgramName, sEventName, vSoundProperties);
    }

    public Vector getSoundProperties(String sProgramName, String sEventName)
    {
        return objRM.getSoundProperties(sProgramName, sEventName);
    }

    public Vector getEventSoundMappings(String sProgramName)
    {
        return objRM.getEventSoundMappings(sProgramName);
    }

    public boolean setNetworkInfo(String sConfServerIPAddress, String sConfServerPortNo, String sListenerIPAddress, String sListenerPortNo)
    {
        return objLC.setNetworkInfo(sConfServerIPAddress, sConfServerPortNo, sListenerIPAddress, sListenerPortNo);
    }

    public boolean setConfigServerInfo(String sConfServerIPAddress, String sConfServerPortNo)
    {
        return objLC.setConfigServerInfo(sConfServerIPAddress, sConfServerPortNo);
    }

    public boolean setListenerInfo(String sListenerIPAddress, String sListenerPortNo)
    {
        return objLC.setListenerInfo(sListenerIPAddress, sListenerPortNo);
    }

    public String[] getConfigServerInfo()
    {
        return objLC.getConfigServerInfo();
    }

    public String[] getListenerInfo()
    {
        return objLC.getListenerInfo();
    }


    public void initialiseRecorders()
    {
        try
        {
            Vector vVector=retrieveRecordedProgramsList();
            if(vVector!=null)
            {
                for(int iLoop=0; iLoop < vVector.size(); iLoop++)
                {
                    String sProgramName = (String) vVector.elementAt(iLoop);
                    htRecordedPrograms.put(sProgramName,new Recorder(sProgramName));
                }
            }
        }
        catch(Exception e)
        {
            //e.printStackTrace();
            System.out.println("RECOREDER UNABLE TO FIND DB");
        }
    }

    /**
     * to be implemented
     * @return
     */
    public boolean registerListener()
    {
        return objLC.registerListener();
    }

    private boolean writeRecordedProgramsList()
    {

        Enumeration objENUM = htRecordedPrograms.keys();
        Vector vProgramsList = new Vector();
         System.out.println("   REcorded");
        while(objENUM.hasMoreElements()==true)
        {
            vProgramsList.add(objENUM.nextElement());
        }

        Database objDB = new Database();

        return objDB.writeObject(RECORDED_PROGRAMS_LIST+RECORDED_PROGRAMS_LIST_FILE_EXTENSION, vProgramsList);
    }

    private Vector retrieveRecordedProgramsList()
    {
        Database objDB = new Database();
        try
        {
            return (Vector)objDB.readObject(RECORDED_PROGRAMS_LIST+RECORDED_PROGRAMS_LIST_FILE_EXTENSION);
        }
        catch(Exception e)
        {
            return null;
        }
    }

    public Vector[] loginListener(String sIPAddress, String sPort)
    {
        Vector vProgramList = objLC.loginListener(sIPAddress, sPort);
        if(vProgramList == null)
        {
            System.out.println("Null returned by ListenerManager.loginListener");
            return null;
        }
        Vector vArray[]= new Vector[2];
        vArray[0]=objRM.getProgramNames();
        /*
        vArray[0] contains Registered Programs
        vArray[1] contains UnRegistered Programs
        */
        vArray[1]= new Vector();
        for(int iLoop=0; iLoop < vProgramList.size(); iLoop++)
        {
            String sProgramName = (String)vProgramList.get(iLoop);
            if(vArray[0].contains(sProgramName) == false)
            {
                vArray[1].add(sProgramName);
            }
        }
        return vArray;
    }
    public boolean logoutListener(String sIPAddress, String sPort)
    {
        return objLC.logoutListener(sIPAddress, sPort);
    }
    /**
     * to be implemented
     * @return
     */
    public boolean unRegisterListener()
    {
        boolean bUnregisterStatus = false;
        bUnregisterStatus = objLC.unRegisterListener();
        if(bUnregisterStatus == true)
        {
            return true;
        }
        else
        {
            return false;
        }

    }

    /**
     * to be implemented
     * @param sProgramName
     * @return
     */
    public boolean registerProgram(String sProgramName)
    {
        System.out.println("Register Program Called for Program : " + sProgramName);
        return objRM.registerProgram(sProgramName, objLC.registerProgram(sProgramName));

    }

    /**
     * to be implemented
     * @param sProgramName
     * @return
     */
    public boolean unRegisterProgram(String sProgramName)
    {
        System.out.println("UnRegister Program Called for Program : " + sProgramName);
        if(objLC.unRegisterProgram(sProgramName) == true)
            return objRM.unRegisterProgram(sProgramName);
        return false;
    }

    public boolean playRecordedSound(String sProgramName)
    {
        Database objDB= new Database();
        try
        {
            vRecordedVector=(Vector[])objDB.readObject(RECORDED_PROGRAM+sProgramName+RECORDED_PROGRAM_FILE_EXTENSION);

            if(vRecordedVector == null)
            {
                System.out.println("Recorded Vector : NULL in playRecordedSound");
                return false;
            }
            bPlayRecordedProgram=true;
            sRecordedProgramName=sProgramName;
            playEvents();
        }
        catch(Exception e)
        {
            System.out.println("EXCEPTION === PLAY RECORDED SOUND");
            return false;
        }
        return true;
    }

    public void startRecording(String sProgramName)
    {
        htRecordedPrograms.put(sProgramName,new Recorder(sProgramName));
        this.writeRecordedProgramsList();
    }

    public void stopRecording(String sProgramName)
    {
        htRecordedPrograms.remove(sProgramName);
        this.writeRecordedProgramsList();
    }

    public void stopPlayingRecordedProgram()
    {
        objSS.stopSound();
    }



    public Vector getRecordedPrograms()
    {
        Enumeration objEnum =  htRecordedPrograms.keys();

        Vector vPrograms = null;

        if(objEnum != null)
        {
            vPrograms= new Vector();
            while(objEnum.hasMoreElements() == true)
            {
                vPrograms.add(objEnum.nextElement());
            }
        }
        return vPrograms;
    }

    public void stopSound()
    {
         objSS.stopSound();
    }

    public void mute(boolean bStatus)
    {
        objSS.mute(bStatus);
    }
    public static Vector getInstrumentNames()
    {
        return objSS.getInstrumentNames();
    }
    public Vector getEventsForProgram(String sProgramName)
    {
       return objRM.getEventSoundHandlers(sProgramName);
    }

    public static  void sendNotify(String sProgramName, EventInfo objEI)
    {
        System.out.println("Send Notify called..");
        vEventsBuffer.add(objEI);
        vProgramsBuffer.add(sProgramName);

        bStatus = true;

         if(htRecordedPrograms.containsKey(sProgramName)==true)
         {
             System.out.println("------------------------------");
            Recorder objRecorder = (Recorder)htRecordedPrograms.get(sProgramName);
            if(objEI.getEventName().equalsIgnoreCase("START") == true)
            {
                System.out.println("Start Event for Recorder-Timer");
                objRecorder.setStartTime();
            }
            else if(objEI.getEventName().equalsIgnoreCase("END") == true)
            {
                System.out.println("STORED RECORDED ********  ");
                objRecorder.storeRecordedProgram();
                htRecordedPrograms.put(sProgramName, new Recorder(sProgramName));
            }
            else
            {
                objRecorder.recordEvent(objEI);
            }

        }
        playEvents();
    }


    public static void playEvents()
    {

        if(bPlayRecordedProgram == true)
        {
            long lPreviousEventTime=0;
            for(int iLoop =0 ; iLoop < vRecordedVector[0].size(); iLoop++)
            {
                long  lCurrentEventTime=((Long)vRecordedVector[0].get(iLoop)).longValue();
                EventInfo objEI=(EventInfo)vRecordedVector[1].get(iLoop);
                try
                {
                    Vector vSoundProperties = objRM.getSoundProperties(sRecordedProgramName, objEI.getEventName());

                    Thread.sleep(lCurrentEventTime-lPreviousEventTime); //compute the difference between 2 events

                    lPreviousEventTime=lCurrentEventTime;     //store the previous event time

                    objSS.playSound(vSoundProperties, objEI.getData());
                }
                catch(InterruptedException ie)
                {
                    System.out.println("SLEEP interrupted FOR PLAYING RECORDED EVENTS");
                }

            }
            bPlayRecordedProgram=false;
            return;
        }

        if(bStatus == true)
        {

            for(int iLoop=0;iLoop< vProgramsBuffer.size(); iLoop++)
            {

                String sProgramName = (String)vProgramsBuffer.remove(0);
                EventInfo objEI = (EventInfo)vEventsBuffer.remove(0);

                Vector vSoundProperties = objRM.getSoundProperties(sProgramName, objEI.getEventName());
             //   System.out.println("Event name==="+objEI.getEventName());
                if(vSoundProperties != null)
                {
                   // System.out.println("Activity STATUS==="+objEI.isActivity());
                    if(objEI.isActivity() == true)
                    {
                      //  System.out.println("ACTIVITY "+objEI.getActivityType());
                        if(objEI.getActivityType().equalsIgnoreCase("START") == true)
                        {
                           // System.out.println("ACTIVITY start");
                            objSS.playActivitiesSound(vSoundProperties);
                        }
                        else if(objEI.getActivityType().equalsIgnoreCase("END") == true)
                        {
                           // System.out.println("Aend");
                            objSS.stopActivitiesSound();
                        }
                    }
                    else
                    {
                        objSS.playSound(vSoundProperties, objEI.getData());
                    }
                }
                else
                {
                   // System.out.println("Sound Properties is Null For  : " + objEI.getEventName());
                }

            }
           // bStatus = false;
        }
    }

}
