package business;

import javax.sound.midi.*;
import java.util.Vector;
import java.util.Hashtable;


public class SoundSynthesizer
{
    private Synthesizer objSynth = null;
    private MidiChannel[] objChannels = null;
    private Hashtable htInstruments = null;
    private Instrument objInstruments[] = null;
    private final int CHANNEL_NUMBER = 0;
    private final int CHANNEL_NUMBER_ACTIVITY = 1;
    private final int DEFAULT_NOTE = 90;
    private final int DEFAULT_NOTE_ACTIVITY = 100;
    private final int VELOCITY=1000;

    public SoundSynthesizer()
    {
        htInstruments = new Hashtable();
        try
        {
                objSynth = MidiSystem.getSynthesizer();
                objSynth.open();
        }
        catch (MidiUnavailableException mue)
        {
          System.out.println("MIDI EXCEPTION ");

        }

        objInstruments = objSynth.getAvailableInstruments();
        objChannels = objSynth.getChannels();
        mapInstrumentNames();

    }
/*
    public void setEventSoundHandler(EventSoundHandler objEH)
    {
        this.objEH = objEH;
    }
*/

    public void playSound(Vector vSoundProperties, Integer objData)
    {
        String sSoundVariable = (String)vSoundProperties.get(0);
        int iPitch = ((Integer)vSoundProperties.get(1)).intValue();
        String sTime = (String)vSoundProperties.get(2);
        boolean bStatus = ((Boolean)(vSoundProperties.get(3))).booleanValue();

        int iData=DEFAULT_NOTE;

        if(objData != null)
        {
            iData=((Integer)objData).intValue();
        }

        int iInstrument=0;
        try{
             iInstrument= ((Integer)htInstruments.get(sSoundVariable)).intValue();
        }catch(Exception e)
        {
           System.out.println("INSTRUMENT NOT FOUND");
        }

        if(bStatus)
        {
             objSynth.loadInstrument(objInstruments[iInstrument]);
             objChannels[CHANNEL_NUMBER].programChange(iInstrument);
             objChannels[CHANNEL_NUMBER].noteOn(iData, VELOCITY); //change constants

            try{
                Thread.sleep(500);
            }
            catch(Exception e)
            {
                System.out.println("EXCEPTION SOUND");
            }

             objChannels[CHANNEL_NUMBER].noteOff(iData);
        }
    }

    public void playActivitiesSound(Vector vSoundProperties)
    {
        String sSoundVariable = (String)vSoundProperties.get(0);
        int iPitch = ((Integer)vSoundProperties.get(1)).intValue();
        String sTime = (String)vSoundProperties.get(2);
        boolean bStatus = ((Boolean)(vSoundProperties.get(3))).booleanValue();

        int iInstrument=0;
        try{
             iInstrument= ((Integer)htInstruments.get(sSoundVariable)).intValue();
        }catch(Exception e)
        {
           System.out.println("INSTRUMENT NOT FOUND");
        }

        if(bStatus)
        {
             objSynth.loadInstrument(objInstruments[iInstrument]);
             objChannels[CHANNEL_NUMBER_ACTIVITY].programChange(iInstrument);
             objChannels[CHANNEL_NUMBER_ACTIVITY].noteOn(DEFAULT_NOTE_ACTIVITY, VELOCITY); //change constants

            try{
                Thread.sleep(500);
            }
            catch(Exception e)
            {
                System.out.println("EXCEPTION SOUND");
            }
        }
        else
        {
            objChannels[CHANNEL_NUMBER_ACTIVITY].noteOff(DEFAULT_NOTE_ACTIVITY);
        }

    }

    public  Vector getInstrumentNames()
    {
       Vector vInstrument = new Vector();

       for(int iLoop=0; iLoop < objInstruments.length; iLoop++)
       {
            vInstrument.add(objInstruments[iLoop].getName().trim());
       }

        return vInstrument;
    }

    private void mapInstrumentNames()
    {

       for(int iLoop=0; iLoop < objInstruments.length; iLoop++)
       {
            htInstruments.put(objInstruments[iLoop].getName().trim(),new Integer(iLoop));
            //System.out.println("MAP------INSTR-"+htInstruments.get(objInstruments[iLoop].getName())+"--VAL---"+htInstruments.get(objInstruments[iLoop].getName()));

       }

    }

    public void stopSound()
    {
        objChannels[CHANNEL_NUMBER].allNotesOff();
    }

    public void stopActivitiesSound()
    {
        objChannels[CHANNEL_NUMBER_ACTIVITY].allNotesOff();
    }

    public void mute(boolean bStatus)
    {
        objChannels[CHANNEL_NUMBER].setMute(bStatus);
    }
}