package network;

import access.NetworkInfoDB;
import rmi.*;
import business.ListenerManager;

import java.rmi.*;
import java.rmi.server.UnicastRemoteObject;
import java.util.Hashtable;
import java.util.Vector;
import java.net.MalformedURLException;

public class ListenerCommunicator extends UnicastRemoteObject implements ConfServerInterface
{

    private  NetworkInfo objNI = null;
    private  ListenerInterface objLI;

    public ListenerCommunicator() throws RemoteException
    {

    }

    public boolean receiveEvent(String sProgramName, EventInfo objEI) throws RemoteException
    {
        ListenerManager.sendNotify(sProgramName, objEI);
        return true;
    }

    private boolean connectConfServer()
    {
        try
        {
            System.out.println(objNI.getConfServerIPAddress() + ":" + objNI.getConfServerPortNo());
            //objLI = (ListenerInterface)javax.rmi.PortableRemoteObject.narrow(Naming.lookup("rmi://"+ objNI.getConfServerIPAddress() + ":" + objNI.getConfServerPortNo() + "/LISTENER_SERVER"),ListenerInterface.class);
           objLI = (ListenerInterface)Naming.lookup("rmi://"+ objNI.getConfServerIPAddress() + ":" + objNI.getConfServerPortNo() + "/LISTENER_SERVER");


            if(objLI !=null)
            {
                System.out.println("LISTENER CONNECTED");
            }
            else
            {
                System.out.println("Listener object is null");
                System.out.println("LISTENER NOT CONNECTED");
            }
            return true;
        }
        catch(Exception e)
        {
            e.printStackTrace();
            System.out.println("Exception at Listener Interface Communicator");
            return false;
        }
    }


    public Hashtable registerProgram(String sProgramName)
    {
        try
        {
            System.out.println("Register Program of LC");
            return objLI.registerProgram(sProgramName, objNI.getListenerIPAddress(), objNI.getListenerPortNo());
        }
        catch(RemoteException re)
        {
            return null;
        }
    }

    public boolean unRegisterProgram(String sProgramName)
    {
        try
        {
            return objLI.unRegisterProgram(sProgramName, objNI.getListenerIPAddress(), objNI.getListenerPortNo());
        }
        catch(RemoteException re)
        {
            return false;
        }
    }
    public boolean registerListener()
    {
        try
        {
            System.out.println("LI MMMMM"+objLI+"\n  info"+objNI.getListenerIPAddress() + ":" + objNI.getListenerPortNo());
            return objLI.registerListener(objNI.getListenerIPAddress(), objNI.getListenerPortNo());
        }
        catch(RemoteException re)
        {
            System.out.println("EXCEPTION ");
            re.printStackTrace();
            return false;
        }
    }
    public boolean unRegisterListener()
    {
        try
        {
            return objLI.unRegisterListener(objNI.getListenerIPAddress(), objNI.getListenerPortNo());
        }
        catch(RemoteException re)
        {
            return false;
        }
    }


    public boolean initialise()
    {
        if((objNI=getNetworkInfo()) == null)
        {
            System.out.println("inside INIT");
            objNI = new NetworkInfo();
            return false;
        }
        else
        {
            System.out.println("out   INIT");
            establishCommunication();
            return true;
        }

    }

    private boolean startListener()
    {
        /*if(System.getSecurityManager() == null)
        {
            System.setSecurityManager(new RMISecurityManager());
        } */

          try
          {
            Naming.rebind("rmi://"+objNI.getListenerIPAddress()+ ":" + objNI.getListenerPortNo() +"/CONF_SERVER_EVENT_RECEIVER", this);
          }
          catch(Exception ce)
          {
              ce.printStackTrace();
            System.out.println("Error in Connection Establishment of LISTENER");
            return false;
          }
        return true;
    }

    public NetworkInfo getNetworkInfo()
    {
        NetworkInfoDB objNIDB = new NetworkInfoDB();
        return objNIDB.retrieveNetworkInfo();
    }

    public boolean setConfigServerInfo(String sIPAddress, String sPortNo)
    {
        objNI.setConfServerIPAddress(sIPAddress);
        objNI.setConfServerPortNo(sPortNo);

        NetworkInfoDB objNIDB = new NetworkInfoDB();
        return objNIDB.storeNetworkInfo(objNI);
    }

    public boolean setListenerInfo(String sIPAddress, String sPortNo)
    {
        objNI.setListenerIPAddress(sIPAddress);
        objNI.setConfServerPortNo(sPortNo);

        NetworkInfoDB objNIDB = new NetworkInfoDB();
        return objNIDB.storeNetworkInfo(objNI);
    }

    public boolean setNetworkInfo(String sConfServerIPAddress, String sConfServerPortNo, String sListenerIPAddress, String sListenerPortNo)
    {
        System.out.println("CALLED NI UPDATE");
        objNI.setConfServerIPAddress(sConfServerIPAddress);
        objNI.setConfServerPortNo(sConfServerPortNo);
        objNI.setListenerIPAddress(sListenerIPAddress);
        objNI.setListenerPortNo(sListenerPortNo);

        establishCommunication();

        NetworkInfoDB objNIDB = new NetworkInfoDB();
        return objNIDB.storeNetworkInfo(objNI);
     }

    public String[] getConfigServerInfo()
    {
        String objStr[]=new String[2];
        objStr[0]=objNI.getConfServerIPAddress();
        objStr[1]=objNI.getConfServerPortNo();
        return objStr;
    }

    public String[] getListenerInfo()
    {
        String objStr[]=new String[2];
        objStr[0]=objNI.getListenerIPAddress();
        objStr[1]=objNI.getListenerPortNo();
        return objStr;
    }



     public boolean establishCommunication()
     {
         if(startListener()== false)
         {
             System.out.println("START LISTENER : FALSE");
             return false;
         }
         else
         {
            if(connectConfServer() == false)
            {
                System.out.println("CONNECT CONF SERVER : FALSE");
                return false;
            }
            else
            {
                System.out.println("CONNECT CONF SERVER : TRUE");
                return true;
            }
         }
     }


    public Vector loginListener(String sIPAddress, String sPort)
    {
       try
        {
            return objLI.loginListener(sIPAddress, sPort);
        }
        catch(RemoteException re)
        {
            return null;
        }
    }

    public boolean logoutListener(String sIPAddress, String sPort)
    {
        try
        {
           if(objLI.logoutListener(sIPAddress, sPort) == false)
           {
               return false;
           }
           else
           {
                Naming.unbind("rmi://"+objNI.getListenerIPAddress()+ ":" + objNI.getListenerPortNo() +"/CONF_SERVER_EVENT_RECEIVER");
                return true;
           }
        }
        catch(RemoteException e)
        {
            return false;
        }
        catch(MalformedURLException me)
        {
            System.out.println("Malformed URL EXCEPTION");
            return false;
         }
        catch(NotBoundException nbe)
        {
            System.out.println("NOT BOUND EXCEPTION");
            return false;
        }
    }
}